<?php

namespace NoccyLabs\Hotfix\Command;

use Symfony\Component\Console\Command\Command;

use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Question\ConfirmationQuestion;
use Symfony\Component\Console\Question\ChoiceQuestion;
use Symfony\Component\ExpressionLanguage\ExpressionLanguage;
use NoccyLabs\Hotfix\Hotfix\Loader;
use NoccyLabs\Hotfix\System\Facts;


class ApplyCommand extends Command
{

    protected function configure()
    {
        $this->setName("apply");
        $this->setDescription("Apply a hotfix from a file or the Internet");

        $this->addOption("insecure", "I", InputOption::VALUE_NONE, "Disable signature checks. Don't use unless you know what you are doing!");
        $this->addOption("preview", "p", InputOption::VALUE_NONE, "Only preview the hotfix script, don't apply anything");

        $this->addArgument("hotfix", InputArgument::OPTIONAL, "The identifier, path or filename of the hotfix");
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $this->output = $output;

        $fix = $input->getArgument("hotfix");
        $insecure = $input->getOption("insecure");
        $loader = new Loader();

        if (!$fix) {
        
            $loaders = $loader->getLoaders();
            $output->writeln("Supported loaders:\n");
            foreach ($loaders as $loader) {
                $output->writeln(" * ".$loader->getInfo());
            }
            return;
        }

        $output->writeln("Reading hotfix <comment>{$fix}</comment>...");

        try {
            $hotfix = $loader->load($fix, $insecure);
        } catch (\Exception $e) {
            $output->writeln("<error>Error: ".$e->getMessage()."</error>");
            return;
        }
        if (!$hotfix) {
            $output->writeln("<error>Could not load hotfix</error>");
            return;
        }

        $output->writeln("");
        if (($signer = $hotfix->getSignedBy())) {
            $keyid = $hotfix->getKeyId();
            $output->writeln("Hotfix has good signature from <fg=green;options=bold>{$signer}</fg=green;options=bold> (keyid <info>{$keyid}</info>)");
        } else {
            $output->writeln("<fg=red;options=bold>Warning: Hotfix is not signed or signature not valid!</fg=red;options=bold>");
        }
        
        $requires = $hotfix->getRequirements();
        if (count($requires)>0) {
            $engine = new ExpressionLanguage();
            $facts = Facts::getSystemFacts()->getFacts();
            while (true) {
                $expr = array_shift($requires);
                $ret = $engine->evaluate($expr, $facts);
                if ($ret) {
                    //$output->writeln("<fg=green;options=bold>Hotfix is compatible with the current distribution</>");
                    break;
                }
                if (count($requires)==0) {
                    $output->writeln("<error>Error: This hotfix it not compatible with the current distribution</>");
                    return false;
                }
            }
        //} else {
            //$output->writeln("<fg=green;options=bold>Hotfix indicates universal compatibility.</>");
        }


        $output->writeln("");
        $output->writeln("  Hotfix:  <fg=yellow;options=bold>".$hotfix->getName()."</fg=yellow;options=bold>");
        $output->writeln("  Author:  <comment>".$hotfix->getAuthor()."</comment>");
        $info = explode("\n",wordwrap(trim($hotfix->getInfo()), 60));
        $info = "<comment>".join("</comment>\n           <comment>", $info)."</comment>";
        $output->writeln("  Info:    ".$info);
        $output->writeln("");
        
        if ($input->getOption("preview")) {
            $output->writeln("This is the script that will be executed:");
            $body = $hotfix->getBody();
            $body = "<fg=cyan>  ".join("\n  ", explode("\n",$body))."</fg=cyan>";
            $output->writeln($body);
            return;
        }
        
        $helper = $this->getHelper("question");
        $question = new ConfirmationQuestion("Do you want to apply this hotfix? [y/N] ", false);
        
        if (!$helper->ask($input, $output, $question)) {
            return;
        }
        
        $output->writeln("\n<info>Applying hotfix...</info>\n");
        try {
            $hotfix->apply();
        } catch (\Exception $e) {
            $output->writeln("\n<error>Error: ".$e->getMessage()."</error>");
        }
        $output->writeln("\n<info>Hotfix applied successfully</info>");
    }

}
